@@ -17,7 +17,6 @@
  */
 
 #include "browserrun.h"
-#include "browserrun_p.h"
 
 #include "browseropenorsavequestion.h"
 
@@ -42,7 +41,7 @@
 
 using namespace KParts;
 
-class BrowserRun::BrowserRunPrivate
+class Q_DECL_HIDDEN BrowserRun::BrowserRunPrivate
 {
 public:
     bool m_bHideErrorDialog;
@@ -113,21 +112,23 @@ void BrowserRun::init()
 
 void BrowserRun::scanFile()
 {
-    // qDebug() << KRun::url();
+    const QUrl url = KRun::url();
+    // qDebug() << url;
 
     // Let's check for well-known extensions
     // Not when there is a query in the URL, in any case.
     // Optimization for http/https, findByURL doesn't trust extensions over http.
-    QString protocol = KRun::url().scheme();
+    QString protocol = url.scheme();
 
     if (!KProtocolInfo::proxiedBy(protocol).isEmpty()) {
         QString dummy;
-        protocol = KProtocolManager::slaveProtocol(KRun::url(), dummy);
+        protocol = KProtocolManager::slaveProtocol(url, dummy);
     }
 
-    if (!KRun::url().hasQuery() && !protocol.startsWith(QLatin1String("http"))) {
+    if (!url.hasQuery() && !protocol.startsWith(QLatin1String("http")) &&
+           (!url.path().endsWith(QLatin1Char('/')) || KProtocolManager::supportsListing(url))) {
         QMimeDatabase db;
-        QMimeType mime = db.mimeTypeForUrl(KRun::url());
+        QMimeType mime = db.mimeTypeForUrl(url);
         if (!mime.isDefault() || isLocalFile()) {
             // qDebug() << "MIME TYPE is" << mime.name();
             mimeTypeDetermined(mime.name());
@@ -155,11 +156,11 @@ void BrowserRun::scanFile()
     }
 
     KIO::TransferJob *job;
-    if (d->m_browserArgs.doPost() && KRun::url().scheme().startsWith(QLatin1String("http"))) {
-        job = KIO::http_post(KRun::url(), d->m_browserArgs.postData, KIO::HideProgressInfo);
+    if (d->m_browserArgs.doPost() && url.scheme().startsWith(QLatin1String("http"))) {
+        job = KIO::http_post(url, d->m_browserArgs.postData, KIO::HideProgressInfo);
         job->addMetaData(QStringLiteral("content-type"), d->m_browserArgs.contentType());
     } else {
-        job = KIO::get(KRun::url(),
+        job = KIO::get(url,
                        d->m_args.reload() ? KIO::Reload : KIO::NoReload,
                        KIO::HideProgressInfo);
     }
@@ -187,7 +188,7 @@ void BrowserRun::slotBrowserScanFinished(KJob *job)
         // qDebug() << "It is in fact a directory!";
         // Update our URL in case of a redirection
         KRun::setUrl(static_cast<KIO::TransferJob *>(job)->url());
-        setJob(0);
+        setJob(nullptr);
         mimeTypeDetermined(QStringLiteral("inode/directory"));
     } else {
         KRun::slotScanFinished(job);
@@ -216,7 +217,7 @@ void BrowserRun::slotBrowserMimetype(KIO::Job *_job, const QString &type)
     if (job->isErrorPage()) {
         d->m_mimeType = type;
         handleError(job);
-        setJob(0);
+        setJob(nullptr);
     } else {
         // qDebug() << "found" << type << "for" << KRun::url();
 
@@ -243,7 +244,7 @@ void BrowserRun::slotBrowserMimetype(KIO::Job *_job, const QString &type)
         // Make a copy to avoid a dead reference
         QString _type = type;
         job->putOnHold();
-        setJob(0);
+        setJob(nullptr);
 
         // If the current mime-type is the default mime-type, then attempt to
         // determine the "real" mimetype from the file name.
@@ -258,7 +259,7 @@ void BrowserRun::slotBrowserMimetype(KIO::Job *_job, const QString &type)
 
 BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable(const QString &mimeType)
 {
-    return handleNonEmbeddable(mimeType, NULL);
+    return handleNonEmbeddable(mimeType, nullptr);
 }
 
 BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable(const QString &_mimeType, KService::Ptr *selectedService)
@@ -340,17 +341,17 @@ bool BrowserRun::allowExecution(const QString &mimeType, const QUrl &url)
         return false;
     }
 
-    return (KMessageBox::warningContinueCancel(0,
+    return (KMessageBox::warningContinueCancel(nullptr,
             i18n("Do you really want to execute '%1'?", url.toDisplayString()),
             i18n("Execute File?"), KGuiItem(i18n("Execute"))) == KMessageBox::Continue);
 }
 
 //static, deprecated
-#ifndef KPARTS_NO_DEPRECATED
+#if KPARTS_BUILD_DEPRECATED_SINCE(5, 0)
 BrowserRun::AskSaveResult BrowserRun::askSave(const QUrl &url, KService::Ptr offer, const QString &mimeType, const QString &suggestedFileName)
 {
     Q_UNUSED(offer);
-    BrowserOpenOrSaveQuestion question(0, url, mimeType);
+    BrowserOpenOrSaveQuestion question(nullptr, url, mimeType);
     question.setSuggestedFileName(suggestedFileName);
     const BrowserOpenOrSaveQuestion::Result result = question.askOpenOrSave();
     return result == BrowserOpenOrSaveQuestion::Save ? Save
@@ -360,14 +361,14 @@ BrowserRun::AskSaveResult BrowserRun::askSave(const QUrl &url, KService::Ptr off
 #endif
 
 //static, deprecated
-#ifndef KPARTS_NO_DEPRECATED
+#if KPARTS_BUILD_DEPRECATED_SINCE(5, 0)
 BrowserRun::AskSaveResult BrowserRun::askEmbedOrSave(const QUrl &url, const QString &mimeType, const QString &suggestedFileName, int flags)
 {
-    BrowserOpenOrSaveQuestion question(0, url, mimeType);
+    BrowserOpenOrSaveQuestion question(nullptr, url, mimeType);
     question.setSuggestedFileName(suggestedFileName);
     const BrowserOpenOrSaveQuestion::Result result = question.askEmbedOrSave(flags);
     return result == BrowserOpenOrSaveQuestion::Save ? Save
-           : BrowserOpenOrSaveQuestion::Embed ? Open
+           : result == BrowserOpenOrSaveQuestion::Embed ? Open
            : Cancel;
 }
 #endif
@@ -378,12 +379,14 @@ void BrowserRun::save(const QUrl &url, const QString &suggestedFileName)
     saveUrl(url, suggestedFileName, d->m_window, d->m_args);
 }
 
+#if KPARTS_BUILD_DEPRECATED_SINCE(4, 4)
 // static
 void BrowserRun::simpleSave(const QUrl &url, const QString &suggestedFileName,
                             QWidget *window)
 {
     saveUrl(url, suggestedFileName, window, KParts::OpenUrlArguments());
 }
+#endif
 
 void KParts::BrowserRun::saveUrl(const QUrl &url, const QString &suggestedFileName,
                                  QWidget *window, const KParts::OpenUrlArguments &args)
@@ -402,7 +405,7 @@ void KParts::BrowserRun::saveUrl(const QUrl &url, const QString &suggestedFileNa
             if (cmd.isEmpty()) {
                 QString errMsg = i18n("The Download Manager (%1) could not be found in your $PATH ", downloadManger);
                 QString errMsgEx = i18n("Try to reinstall it  \n\nThe integration with Konqueror will be disabled.");
-                KMessageBox::detailedSorry(0, errMsg, errMsgEx);
+                KMessageBox::detailedSorry(nullptr, errMsg, errMsgEx);
                 cfg.writePathEntry("DownloadManager", QString());
                 cfg.sync();
             } else {
@@ -428,7 +431,7 @@ void KParts::BrowserRun::saveUrl(const QUrl &url, const QString &suggestedFileNa
     QFileDialog *dlg = new QFileDialog(window);
     dlg->setAcceptMode(QFileDialog::AcceptSave);
     dlg->setWindowTitle(i18n("Save As"));
-    dlg->setConfirmOverwrite(true);
+    dlg->setOption(QFileDialog::DontConfirmOverwrite, false);
 
     QString name;
     if (!suggestedFileName.isEmpty()) {
@@ -460,8 +463,7 @@ void BrowserRun::saveUrlUsingKIO(const QUrl &srcUrl, const QUrl &destUrl,
     job->addMetaData(QStringLiteral("MaxCacheSize"), QStringLiteral("0")); // Don't store in http cache.
     job->addMetaData(QStringLiteral("cache"), QStringLiteral("cache")); // Use entry from cache if available.
     KJobWidgets::setWindow(job, window);
-    job->ui()->setAutoErrorHandlingEnabled(true);
-    new DownloadJobWatcher(job, metaData);
+    job->uiDelegate()->setAutoErrorHandlingEnabled(true);
 }
 
 void BrowserRun::handleError(KJob *job)
@@ -476,7 +478,7 @@ void BrowserRun::handleError(KJob *job)
         // The default handling of error pages is to show them like normal pages
         // But this is done here in handleError so that KHTMLRun can reimplement it
         tjob->putOnHold();
-        setJob(0);
+        setJob(nullptr);
         if (!d->m_mimeType.isEmpty()) {
             mimeTypeDetermined(d->m_mimeType);
         }
@@ -524,7 +526,7 @@ void BrowserRun::redirectToError(int error, const QString &errorText)
      * an error:/ URL that sends the info to khtml.
      */
     KRun::setUrl(makeErrorUrl(error, errorText, url()));
-    setJob(0);
+    setJob(nullptr);
     mimeTypeDetermined(QStringLiteral("text/html"));
 }
 
@@ -534,7 +536,7 @@ void BrowserRun::slotCopyToTempFileResult(KJob *job)
         job->uiDelegate()->showErrorMessage();
     } else {
         // Same as KRun::foundMimeType but with a different URL
-        (void)(KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), d->m_mimeType, d->m_window));
+        (void)(KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), d->m_mimeType, d->m_window, KRun::RunFlags(KRun::RunExecutables)));
     }
     setError(true);   // see above
     setFinished(true);
@@ -574,4 +576,3 @@ KParts::BrowserArguments &KParts::BrowserRun::browserArguments()
 }
 
 #include "moc_browserrun.cpp"
-#include "moc_browserrun_p.cpp"
